import { DeviceType } from '@ucap-webmessenger/core';
import {
  APIRequest,
  APIResponse,
  APIEncoder,
  APIDecoder,
  ParameterUtil
} from '@ucap-webmessenger/api';

export interface MassTalkDownloadRequest extends APIRequest {
  userSeq: number;
  deviceType: DeviceType;
  token: string;
  eventMassSeq?: number;
}

export interface MassTalkDownloadResponse extends APIResponse {
  content?: string;
  userName?: string;
  regDate?: string;
}

const massTalkDownloadEncodeMap = {
  userSeq: 'p_user_seq',
  deviceType: 'p_device_type',
  token: 'p_token',
  eventMassSeq: 'p_event_mass_seq'
};

export const encodeMassTalkDownload: APIEncoder<MassTalkDownloadRequest> = (
  req: MassTalkDownloadRequest
) => {
  return ParameterUtil.encode(massTalkDownloadEncodeMap, req);
};

export const decodeMassTalkDownload: APIDecoder<MassTalkDownloadResponse> = (
  res: any
) => {
  return {
    statusCode: res.StatusCode,
    errorMessage: res.ErrorMessage,
    content: res.Content,
    userName: res.UserName,
    regDate: res.RegDate
  } as MassTalkDownloadResponse;
};
