import { Component, OnInit, Output, EventEmitter, Input } from '@angular/core';
import { ucapAnimations } from '@ucap-webmessenger/ui';
import { Observable } from 'rxjs';
import {
  DeptInfo,
  QueryProtocolService,
  DeptSearchType,
  UserInfoSS,
  DeptUserResponse,
  UserInfoF,
  UserInfoDN
} from '@ucap-webmessenger/protocol-query';
import { UserInfo } from '@ucap-webmessenger/protocol-sync';
import { Store, select } from '@ngrx/store';
import { NGXLogger } from 'ngx-logger';

import * as AppStore from '@app/store';
import * as QueryStore from '@app/store/messenger/query';
import { SessionStorageService } from '@ucap-webmessenger/web-storage';
import { LoginInfo, KEY_LOGIN_INFO } from '@app/types';
import { take, map, switchMap } from 'rxjs/operators';
import { VersionInfo2Response } from '@ucap-webmessenger/api-public';
import { KEY_VER_INFO } from '@app/types/ver-info.type';

@Component({
  selector: 'app-layout-chat-left-sidenav-organization',
  templateUrl: './organization.component.html',
  styleUrls: ['./organization.component.scss'],
  animations: ucapAnimations
})
export class OrganizationComponent implements OnInit {
  @Input()
  /** 사용자 선택용으로 사용시 true 로 유입 */
  isUserSelect = false;
  @Input()
  /** 선택된 사용자의 리스트 */
  selectedUserList?: (UserInfo | UserInfoSS | UserInfoF | UserInfoDN)[] = [];

  @Output()
  checkUser = new EventEmitter<{
    isChecked: boolean;
    userInfo: UserInfo | UserInfoSS | UserInfoF | UserInfoDN;
  }>();

  departmentInfoList$: Observable<DeptInfo[]>;
  selectedDepartmentUserInfoList$: Observable<UserInfoSS[]>;
  selectedDepartmentStatus$: Observable<DeptUserResponse>;

  loginInfo = this.sessionStorageService.get<LoginInfo>(KEY_LOGIN_INFO);
  sessionVerinfo = this.sessionStorageService.get<VersionInfo2Response>(
    KEY_VER_INFO
  );

  constructor(
    private store: Store<any>,
    private queryProtocolService: QueryProtocolService,
    private sessionStorageService: SessionStorageService,
    private logger: NGXLogger
  ) {}

  ngOnInit() {
    this.departmentInfoList$ = this.store.pipe(
      select(AppStore.MessengerSelector.QuerySelector.departmentInfoList)
    );
    this.selectedDepartmentUserInfoList$ = this.store.pipe(
      select(
        AppStore.MessengerSelector.QuerySelector.selectedDepartmentUserInfoList
      )
    );
    this.selectedDepartmentStatus$ = this.store.pipe(
      select(AppStore.MessengerSelector.QuerySelector.selectedDepartmentStatus)
    );

    this.store
      .pipe(
        take(1),
        select(AppStore.AccountSelector.AuthenticationSelector.loginRes),
        map(loginRes => {
          this.store.dispatch(
            QueryStore.deptUser({
              divCd: 'ORG',
              companyCode: this.loginInfo.companyCode,
              seq: loginRes.departmentCode,
              search: '',
              searchRange: DeptSearchType.All,
              senderCompanyCode: this.loginInfo.companyCode,
              senderEmployeeType: loginRes.userInfo.employeeType
            })
          );
        })
      )
      .subscribe();
  }

  onSelectedOrganization(deptInfo: DeptInfo) {
    this.store
      .pipe(
        take(1),
        select(AppStore.AccountSelector.AuthenticationSelector.loginRes),
        map(loginRes => {
          this.store.dispatch(
            QueryStore.deptUser({
              divCd: 'ORG',
              companyCode: this.loginInfo.companyCode,
              seq: deptInfo.seq,
              search: '',
              searchRange: DeptSearchType.All,
              senderCompanyCode: this.loginInfo.companyCode,
              senderEmployeeType: loginRes.userInfo.employeeType
            })
          );
        })
      )
      .subscribe();
  }

  /** 리스트 checkable 할 경우 checkbox 의 isChecked 를 관장하며 리스트의 전체선택 여부를 판단한다. */
  getCheckedUser(userInfo: UserInfo | UserInfoSS | UserInfoF | UserInfoDN) {
    if (!!this.selectedUserList && this.selectedUserList.length > 0) {
      return (
        this.selectedUserList.filter(item => item.seq === userInfo.seq).length >
        0
      );
    }
    return false;
  }

  /** 리스트가 checkable 할 경우 checkbox 의 change 이벤트를 상위 컴포넌트로 전달한다. */
  onCheckUser(params: {
    isChecked: boolean;
    userInfo: UserInfo | UserInfoSS | UserInfoF | UserInfoDN;
  }) {
    this.checkUser.emit(params);
  }
}
